// ============================================================
// File Name: ovlgen.vhd
// Entity Name(s): ovlgen
// Author(s): Dean Anderson, Kelly Spiller
// Desription:
// Overlay sub-window tracking and download image creation.
// ============================================================
// (c) 2005, Sensoray Co., Inc.
// ============================================================
// FOR SENSORAY'S INTERNAL USE ONLY. NOT FOR DISTRIBUTION 
// OUTSIDE SENSORAY COMPANY, INC.
// ============================================================

#define YCRCB_OUT
/*
#ifndef SENSORAY_OS_LINUX
#include "..\..\mid2246const.h"
#else
#include "../../mid2246const.h"
#endif
*/
#ifdef SENSORAY_OS_LINUX
#include <endian.h>
#endif

#include "ovlgen.h"
#include <string.h> // for memcpy
#include <stdio.h>
#include "allegro.h"
#include "alfont.h"

// Global
int gMaxPix = OVLMAXPIX; // 32768 16-bit pix in FPGA ram or Overlay pix in DDR
static int bOverlayInit = 0;
// Overlay description structure. 
typedef struct _OVLDESC
{
    int             XPos        [OVLWINDOWS]; 
    int             YPos        [OVLWINDOWS];
    int             Type        [OVLWINDOWS];
    ALFONT_FONT     *Font       [OVLWINDOWS];
    /* 260 is the usual max path for windows and linux */
    char            last_font   [OVLWINDOWS][260];
    int             Antialiased [OVLWINDOWS];
    int             Color       [OVLWINDOWS];
    int             BkgColor    [OVLWINDOWS];
    BITMAP          *Bmp        [OVLWINDOWS];
    char            *rgb24Ptr   [OVLWINDOWS];
    unsigned short  Bitmap      [OVLMAXPIXBUFSZ];
    int             Interlaced;
} OVLDESC;

// Segment structure
typedef struct _LINESEG
{
    int     SubWin;
    int     OffsetStart;
    int     OffsetEnd;
    int     FrameY;
    int     LocalX;
    int     LocalY;
    int     Type;
    BITMAP  *Bmp;
} LINESEG;


// Compare funtion used by Quicksort.
static int ComparePos( const void *arg1, const void *arg2 )
{
    int rval;
    const LINESEG *LineSeg1 = (const LINESEG *) arg1;
    const LINESEG *LineSeg2 = (const LINESEG *) arg2;
    // Compare Virtual start pixel offset values for each segment.
    rval = LineSeg1->OffsetStart - LineSeg2->OffsetStart;
    return rval;
}

#ifdef YCRCB_OUT
// Color space conversion function
//
// Y  =  0.257R + 0.504G + 0.098B + 16
// Cr =  0.439R - 0.368G - 0.071B + 128
// Cb = -0.148R - 0.291G + 0.439B + 128
//
// or using integer math.
//
// Y  = ( 16842R + 33030G +  6422B + 1048576 ) / 65536
// Cr = ( 28770R - 24117G -  4653B + 8388608 ) / 65536
// Cb = ( -9699R - 19070G + 28770B + 8388608 ) / 65536
//
inline static unsigned short RGB2YCrCb16( int R, int G, int B )
{
    unsigned short res;
    if ( R == 0 && G == 0 && B == 0 )
        return 0;

    int Y  = ( 16842 * R + 33030 * G +  6422 * B + 1048576 ) >> ( 16 + 2 );
    int Cr = ( 28770 * R - 24117 * G -  4653 * B + 8388608 ) >> ( 16 + 3 );
    int Cb = ( -9699 * R - 19070 * G + 28770 * B + 8388608 ) >> ( 16 + 3 );

    Y  = Y  > ( 235 >> 2 ) ? ( 235 >> 2 ) : Y;
    Cr = Cr > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cr;
    Cb = Cb > ( 240 >> 3 ) ? ( 240 >> 3 ) : Cb;

    Y  = Y  < ( 16 >> 2 ) ? ( 16 >> 2 ) : Y;
    Cr = Cr < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cr;
    Cb = Cb < ( 16 >> 3 ) ? ( 16 >> 3 ) : Cb;
    res = ( Y << 10 ) + ( Cr << 5 ) + Cb;
#if __BYTE_ORDER == __BIG_ENDIAN
    res = ((res >> 8) & 0xff) | ((res & 0xff) << 8);
#endif
    return res;
}
#endif //YCRCB_OUT

///////////////////////////////////////////////////////////////////////
// Create overlay system. This must be called first as it returns a
// handle used by all the other overlay functions.
//
// Interlaced   - Non-zero if overlay buffer is interlaced.
//
// Returns handle on success, null on failure. 
//
OVLHAN DLLEXP CreateOverlay( int Interlaced )
{
    OVLDESC *OvlDesc;
    int i;

    // Initializing Allegro
    if( !bOverlayInit) {
        allegro_init();

        // Initializing AllegroFont
        if ( alfont_init() != ALFONT_OK ) 
        {
            printf("alfont_init failed!\n");
            allegro_exit();
            return NULL;
        }
        bOverlayInit = 1;
    }

    // Allocate memory for the overlay description structure.
    OvlDesc = (OVLDESC *)malloc( sizeof(OVLDESC) );

    // If allocation successful, clear the structure and set the global
    // color depth.
    if ( OvlDesc )
    {
        // Set some defaults...
        for ( i = 0; i < OVLWINDOWS; i++ )
        {
            OvlDesc->Font       [i] = 0;        // None loaded.
            OvlDesc->Bmp        [i] = 0;        // None loaded. 
            OvlDesc->rgb24Ptr   [i] = 0;        // None loaded. 
            OvlDesc->Color      [i] = 0x7fff;   // Set to white
            OvlDesc->BkgColor   [i] = -1;       // backgroud transparent
            OvlDesc->Type       [i] =  1;       // 0=Text, 1=Image
        }
        set_color_depth( 16 );
        OvlDesc->Interlaced = Interlaced;
    } else {
        printf("ovldesc malloc failed!\n");
    }

    // The pointer to the overlay description structure becomes our
    // handle.
    return OvlDesc;
}

///////////////////////////////////////////////////////////////////////
// Delete overlay sub-window position
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 to OVLWINDOWS-1.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP DelOverlayIndex( OVLHAN OvlHandle, int WinIndex)
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    if ( OvlDesc->Bmp[WinIndex] ) {
        destroy_bitmap( OvlDesc->Bmp[WinIndex] );
    }
    if ( OvlDesc->Font[WinIndex] ) {
        alfont_destroy_font( OvlDesc->Font[WinIndex] );
    }
    if ( OvlDesc->rgb24Ptr[WinIndex] ) {
        free( OvlDesc->rgb24Ptr[WinIndex] );
        OvlDesc->rgb24Ptr[WinIndex] = 0;
    }

    OvlDesc->Font       [WinIndex] = 0;        // None loaded.
    OvlDesc->Bmp        [WinIndex] = 0;        // None loaded. 
    OvlDesc->rgb24Ptr   [WinIndex] = 0;        // None loaded. 
    OvlDesc->Color      [WinIndex] = 0x7fff;   // Set to white
    OvlDesc->BkgColor   [WinIndex] = -1;       // backgroud transparent
    OvlDesc->Type       [WinIndex] =  1;       // 0=Text, 1=Image

    // Return success.
    return 0;
}

///////////////////////////////////////////////////////////////////////
// Set overlay sub-window position. The values are the same as the values
// given to the hardware overlay generator.
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
// XPos         - Horizontal pixel position from left.
// YPos         - Vertical pixel position from top.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayPosition( OVLHAN OvlHandle, int WinIndex, int XPos, int YPos )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    // Make sure the overlay system has been created and the WinIndex is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    // Save position.
    OvlDesc->XPos[WinIndex] = XPos;
    OvlDesc->YPos[WinIndex] = YPos;

    // Return success.
    return 0;
}


///////////////////////////////////////////////////////////////////////
// Set overlay sub-window font. Call before SetOverlayText() if font
// change desired.
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
// FontFile     - Full name/path to font file.
// FontSize     - Height of font in pixels.
// Antialiased  - Non-zero means antialiased.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayFont( OVLHAN OvlHandle, int WinIndex, char *FontFile, int FontSize, int Antialiased )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;


    if (OvlDesc->Font[WinIndex]) {
        /* If font already exists, check first if it is the same font
         * before destroying it and reloading it.
         * alfont_load_font takes a VERY long time on some machines/OS's.
         */
        if (strncmp(OvlDesc->last_font[WinIndex], FontFile, 260) != 0) {
            alfont_destroy_font( OvlDesc->Font[WinIndex] );
            /* load the new font */
            OvlDesc->Font[WinIndex] = alfont_load_font( FontFile );
            /* save font file */
            strncpy(OvlDesc->last_font[WinIndex], FontFile, 260);
        }
    } else {
        // Create font from font file.
        OvlDesc->Font[WinIndex] = alfont_load_font( FontFile );
	//alfont_set_language(OvlDesc->Font[WinIndex], NULL);
        /* save font file */
        strncpy(OvlDesc->last_font[WinIndex], FontFile, 260);
    }

    // Set font size.
    if ( OvlDesc->Font[WinIndex] )
        alfont_set_font_size( OvlDesc->Font[WinIndex], FontSize );

    // Set aliasing.
    OvlDesc->Antialiased[WinIndex] = Antialiased;

    // Set return success or failure.
    return OvlDesc->Font[WinIndex] ? 0 : -1;
}

///////////////////////////////////////////////////////////////////////
// Set overlay sub-window text colors. Call before SetOverlayText() if
// color change desired.
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
// Color        - Text color.
// BkgColor     - Background color, zero or less means transparent.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayTextColor( OVLHAN OvlHandle, int WinIndex, int Color, int BkgColor )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    // Convert 24-bit color to 15 bit color and save.
    OvlDesc->Color[WinIndex]      = makecol16( ( Color    >> 0 ) & 0xff, ( Color    >> 8 ) & 0xff, ( Color    >> 16 ) & 0xff );
    OvlDesc->BkgColor[WinIndex]   = BkgColor > 0 ? 
                                  makecol16( ( BkgColor >> 0 ) & 0xff, ( BkgColor >> 8 ) & 0xff, ( BkgColor >> 16 ) & 0xff )
                                  : -1;
    // Return success.
    return 0;
}

///////////////////////////////////////////////////////////////////////
// Raterize text to an overlay sub-window.
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
// Text         - Null terminated string.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayText( OVLHAN OvlHandle, int WinIndex, char *Text )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    // Make sure a font has been created.
    if ( !OvlDesc->Font[WinIndex] )
        return -1;

    // If a bitmap already exists, destroy it so that we can create a new one.
    if ( OvlDesc->Bmp[WinIndex] )
        destroy_bitmap( OvlDesc->Bmp[WinIndex] );

    char *pText;
    char tmpbuffer[MAX_TEXTSIZE];
    char *pLines[MAX_TEXTSIZE];
    int  numlines=1;
    int  Width,LineWidth;
    int  Height,LineHeight;
    int  ii;

    // parse the text into lines
    strncpy( tmpbuffer, Text, MAX_TEXTSIZE);
    tmpbuffer[MAX_TEXTSIZE-1] = 0;

    pText     = tmpbuffer;
    pLines[0] = tmpbuffer;
    Width      = 0;
    Height     = 0;
    LineHeight = alfont_text_height( OvlDesc->Font[WinIndex]);
    while( *pText != '\0') {
        if(*pText == 30) { // Character 30 used as Carriage Return for multi-line text in one region
            *pText = '\0';
            Height = Height + LineHeight;
            LineWidth = alfont_text_length( OvlDesc->Font[WinIndex], pLines[numlines-1] );
            Width = (LineWidth > Width) ? LineWidth : Width; // Max
            pLines[numlines++] = ++pText;
            if(numlines >= MAX_TEXTSIZE)
                break;
            if( *pText == '\0') {
                break;
            }
            if( *pText == 30) {
                continue; // Don't inc pText
            }
        }
        pText++;
    }
    Height = Height + LineHeight;
    LineWidth = alfont_text_length( OvlDesc->Font[WinIndex], pLines[numlines-1] );
    Width = (LineWidth > Width) ? LineWidth : Width; // Max

    // Create bitmap the size of the text sub-window.
    if (Width > 0 ) {
        OvlDesc->Bmp[WinIndex] = create_bitmap( Width+1, Height ); // +1 is a hack to stop the HW from clipping the right most pixel
    } else {
        // Let Width stay at 0, so that when we return, 
        // m_overlay[WinIndex].pos.right == m_overlay[WinIndex].pos.left
        // and the hardware will skip it.
        OvlDesc->Bmp[WinIndex] = create_bitmap( Width, Height );
    }

    // If bitmap creation failed, abort.
    if ( !OvlDesc->Bmp[WinIndex] )
        return -1;

    // If background transparent, clear bitmap.
    if ( OvlDesc->BkgColor[WinIndex] < 0 )
        clear_bitmap( OvlDesc->Bmp[WinIndex] );

    // Rasterize text into bitmap buffer.
    // If there are multiple lines, generate one large bitmap and
    // rasterize each line onto it.
    for (ii=0; ii<numlines ; ii++) {
        if ( OvlDesc->Antialiased[WinIndex] )
            alfont_textout_aa_ex( OvlDesc->Bmp[WinIndex], OvlDesc->Font[WinIndex], pLines[ii],
                                0, ii*LineHeight, OvlDesc->Color[WinIndex], OvlDesc->BkgColor[WinIndex] );
        else
            alfont_textout_ex( OvlDesc->Bmp[WinIndex], OvlDesc->Font[WinIndex], pLines[ii],
                                0, ii*LineHeight, OvlDesc->Color[WinIndex], OvlDesc->BkgColor[WinIndex] );
    }

    OvlDesc->Type[WinIndex] =  0; // 0=Text, 1=Image

    // Return success.
    return 0;
}

///////////////////////////////////////////////////////////////////////
// Place an image in an overlay sub-window.
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - (OVLWINDOWS-1)
// ImageFile    - Full name/path to image file. 24-bit bmp and pcx 
//                files are supported.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayImage( OVLHAN OvlHandle, int WinIndex, char *ImageFile )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    //    BITMAP *src;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    // If a bitmap already exists, destroy it so that we can create a new one.
    if ( OvlDesc->Bmp[WinIndex] )
        destroy_bitmap( OvlDesc->Bmp[WinIndex] );

    // Create bitmap from image file.
    OvlDesc->Bmp[WinIndex] = load_bitmap( ImageFile, NULL ); // Original

    OvlDesc->Type[WinIndex] =  1; // 0=Text, 1=Image
    
    // Return success or failure.
    return OvlDesc->Bmp[WinIndex] ? 0 : -1;
}

///////////////////////////////////////////////////////////////////////
// Texture map an image onto a set of 4 coordinates
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - (OVLWINDOWS-1)
// ImageFile    - Full name/path to image file. 24-bit bmp and pcx 
//                files are supported.
// x1, y1, x2, y2, x3, y3, x4, y4 - Coordinates of the polygon, where
//              - (x1,y1) is upper left corner of the texture 
//              - (x2,y2) is upper right corner of the texture 
//              - (x3,y3) is lower right corner of the texture 
//              - (x4,y4) is lower left corner of the texture 
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayQuad( OVLHAN OvlHandle, int WinIndex, char* ImageFile, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4 )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    BITMAP *texture;
    int NewX, NewY, Width, Height;
    int tx1, ty1, tx2, ty2, tx3, ty3, tx4, ty4;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return -1;

    // Calculate the new position and dimensions of the sub-window that fully contains the quad    
    NewX = x1; 
    if ( x2 < NewX ) NewX = x2;
    if ( x3 < NewX ) NewX = x3;
    if ( x4 < NewX ) NewX = x4;
    NewY = y1; 
    if ( y2 < NewY ) NewY = y2;
    if ( y3 < NewY ) NewY = y3;
    if ( y4 < NewY ) NewY = y4;
    Width = x1; 
    if ( x2 > Width ) Width = x2;
    if ( x3 > Width ) Width = x3;
    if ( x4 > Width ) Width = x4;
    Width = Width - NewX + 1;
    Height = y1; 
    if ( y2 > Height ) Height = y2;
    if ( y3 > Height ) Height = y3;
    if ( y4 > Height ) Height = y4;
    Height = Height - NewY + 1;
    
    // If a bitmap already exists, destroy it so that we can create a new one.
    if ( OvlDesc->Bmp[WinIndex] )
        destroy_bitmap( OvlDesc->Bmp[WinIndex] );
    
    // Create bitmap the size of the quad sub-window. (with padding)
    OvlDesc->Bmp[WinIndex] = create_bitmap( Width, Height );
    // If bitmap creation failed, abort.
    if ( !OvlDesc->Bmp[WinIndex] ) {
        return -1;
    }

    // background should be transparent, clear bitmap.
    clear_bitmap( OvlDesc->Bmp[WinIndex] );

    // Load the texture object    
    texture = load_bitmap( ImageFile, NULL );
    if ( !texture ) {
        return -1;
    }
    
    // Fixed-point conversion macros, used for texture coordinates
#define INT2FIX(n) ((n) << 16)
#define FIX2INT(n) ((n) >> 16)

    // Move coordinates relative to new sub-window, and set up texture coordinates in fixed-point format;
    x1 -= NewX;  y1 -= NewY;  tx1 = 0;                      ty1 = 0; 
    x2 -= NewX;  y2 -= NewY;  tx2 = INT2FIX(texture->w)-1;  ty2 = 0; 
    x3 -= NewX;  y3 -= NewY;  tx3 = INT2FIX(texture->w)-1;  ty3 = INT2FIX(texture->h)-1;
    x4 -= NewX;  y4 -= NewY;  tx4 = 0;                      ty4 = INT2FIX(texture->h)-1;

    // rotate coordinates until y1 is on top
    while ( y1 > 0 ) {
        int tmp;
        tmp = x1; x1 = x2; x2 = x3; x3 = x4; x4 = tmp;
        tmp = y1; y1 = y2; y2 = y3; y3 = y4; y4 = tmp;
        tmp = tx1; tx1 = tx2; tx2 = tx3; tx3 = tx4; tx4 = tmp;
        tmp = ty1; ty1 = ty2; ty2 = ty3; ty3 = ty4; ty4 = tmp;
    }
    
    //printf("SetOverlayQuad %d %d %d %d %d %d %d %d\n", x1,y1,x2,y2,x3,y3,x4,y4);
    {
        int y; // scanline counter
        int ax1, sx1, dx1, cx1, dy1, u1, v1, du1, dv1;  // edge 1
        int ax2, sx2, dx2, cx2, dy2, u2, v2, du2, dv2;  // edge 2
        // For each edge, we have a set of variables:
        // ax: actual x value as it moves along the line from src to dest
        // sx: a signed value 1 that indicates the direction to move the x value along the line
        // dx: delta x, the distance of the x value from src to dest
        // cx: counter for x, used for Bresenham line algorithm
        // u,v: actual values for the texture coordinate as it moves from src to dest
        // du,dv: change vector for the texture coordinate, length is equal to 1 scanline  

        // setup the variables for a particular edge, avoid division by zero in texture vector calculation
#define EDGE_SETUP(edge,src,dest) { \
    ax ## edge = x ## src;              \
    dy ## edge = y ## dest - y ## src;  \
    cx ## edge = 0;                     \
    u ## edge = tx ## src;              \
    v ## edge = ty ## src;              \
    if ( dy ## edge == 0 ) {            \
        dx ## edge = 0;                 \
        sx ## edge = 0;                 \
        du ## edge = 0;                 \
        dv ## edge = 0;                 \
    } else {                            \
        dx ## edge = ( x ## dest > x ## src ) ? x ## dest - x ## src : x ## src - x ## dest;    \
        sx ## edge = ( x ## dest > x ## src ) ? 1 : -1;                                     \
        du ## edge = ( tx ## dest - tx ## src ) / dy ## edge;                                   \
        dv ## edge = ( ty ## dest - ty ## src ) / dy ## edge;                                   \
    }                                   \
}       
        // both edges start at coordinate 1, edge1 heads toward coord 2, edge2 heads toward coord 4
        EDGE_SETUP( 1, 1, 2);
        EDGE_SETUP( 2, 1, 4);
        // start at scanline y1
        y = y1;
        
        // loop scanlines, downward
        while ( y < Height ) {
            // check for a change in direction at a coordinate
            if ( y == y2 && y2 < y3 ) EDGE_SETUP( 1, 2, 3 );  
            if ( y == y4 && y4 < y3 ) EDGE_SETUP( 2, 4, 3 );  
            if ( y == y3 && y3 < y4 ) EDGE_SETUP( 1, 3, 4 );  
            if ( y == y3 && y3 < y2 ) EDGE_SETUP( 2, 3, 2 );
            //printf("1:(%d %d)<%d %d>[%d %d]  2:(%d %d)<%d %d>[%d %d]\n", 
            //  ax1, y, dx1, dy1, u1>>16, v1>>16, ax2, y, dx2, dy2, u2>>16, v2>>16);
            
            if ( y >= 0 ) { 
                // draw a strip of 1 or more pixels
                int x, sx, dx, u, v, du, dv; 
                
                x = ax1;
                sx = ( ax1 < ax2 ) ? 1 : -1;
                dx = ( ax1 < ax2 ) ? ax2 - ax1 : ax1 - ax2;
                u = u1; 
                v = v1;
                du = ( u2 - u1 ) / (dx + 1);
                dv = ( v2 - v1 ) / (dx + 1);
                
                while ( 1 ) {
                    putpixel( OvlDesc->Bmp[WinIndex], x, y, getpixel( texture, FIX2INT(u), FIX2INT(v) ) );
                    if ( x == ax2 ) break;
                    x += sx;
                    u += du;
                    v += dv;
                    //if ( u < 0 || v < 0 || u >= INT2FIX(texture->w) || v >= INT2FIX(texture->h) )
                      //printf( "TEXTURE BOUNDS ERROR: [%d %d]\n", FIX2INT(u), FIX2INT(v) ); 
                }
            }
            
            // calculate new X positions for each edge, using Bresenham line algorithm
            cx1 += dx1;
            while ( cx1 > dy1 ) {
                cx1 -= dy1;
                ax1 += sx1;
            }
            cx2 += dx2;
            while ( cx2 > dy2 ) {
                cx2 -= dy2;
                ax2 += sx2;
            }
            // update the texture positions for each edge
            u1 += du1;
            v1 += dv1;
            u2 += du2;
            v2 += dv2;
            // next scanline
            y++;    
        }
    }

    // Delete the texture image
    destroy_bitmap( texture );

    OvlDesc->Type[WinIndex] =  1; // 0=Text, 1=Image
    OvlDesc->XPos[WinIndex] = NewX;
    OvlDesc->YPos[WinIndex] = NewY;

    // Return success.
    return 0;
}


///////////////////////////////////////////////////////////////////////
// Blit new sub-window to image in overlay zero.
// Assumes OvlDesc->Bmp[0] is loaded into current memory AND double buffer. (Loaded twice)
// Assumes OvlDesc->Bmp[0] is first overlay in memory (Address offset 0)
// Assumes OvlDesc->Bmp[0] has even number of lines
// Assumes OvlDesc->Bmp[0] has a width that is divisible by 128
// Assumes all other overlays (OvlDesc->Bmp[1-7] are contained within the
// boundaries of OvlDesc->Bmp[0]
//
// OvlHandle    - Handle from CreateOverlay().
// OvlBlit      - Pointer to structure that returns with hardware values needed for immediate 'quick' copy.
// ImageFile    - Full name/path to image file. 24-bit bmp and pcx 
//                files are supported.
// XPos         - Horizontal Destination pixel position from left to copy bitmap to.
// YPos         - Vertical Destination pixel position from top to copy bitmap to.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP CopyBmpToOverlayZero( OVLHAN OvlHandle, OVLBLIT *OvlBlit, char *ImageFile, int XPos, int YPos )
{
    unsigned short *OvlBuf;
    OVLDESC        *OvlDesc = (OVLDESC *)OvlHandle;
    BITMAP         *BmpSrc;
    int             OvlPixCnt;
    int             pixval;
    int             LocalX;
    int             LocalY;
    int             BlitX;
    int             BlitYeven;
    int             BlitYodd;
    int             BlitEndX;
    int             BlitW;
    int             BlitH;
    
    if ( !OvlBlit )
        return -1;

    // If bitmap destination invalid
    if ( ! OvlDesc->Bmp[0] ) {
      return -1;
    }

    if ((OvlDesc->Bmp[0]->w % 128) != 0) {
      return -1; // Overlay zero width must be a multiple of 128
    }
    
    // This also assumes that Overlay 0 is the first overlay located
    // in memory, and that all other overlay windows are entirely
    // contained within it.
    //
    // TODO: some sort of check for above.

    // Create bitmap from image file.
    BmpSrc = load_bitmap( ImageFile, NULL ); // bmp, pcx, and tga
    if ( ! BmpSrc ) {
      return(-1);
    }

    // Make sure Sub-Window fits within target window
    // (The blit() routine would clip and work, but the YCrCb convertion routine would not)
    if ( ! ((XPos              >= 0 )                 && (YPos >= 0 ) &&
            (XPos + BmpSrc->w  <= OvlDesc->Bmp[0]->w) && (YPos + BmpSrc->h <= OvlDesc->Bmp[0]->h) ) ) {
      destroy_bitmap( BmpSrc );
      return(-1);
    }

    // If a download hardware bitmap already exists, destroy it so that we can create a new one.
    if (OvlBlit->YCrCb16Bmp)
      free(OvlBlit->YCrCb16Bmp);

    //////////////////////////////////////////////////////////////////////
    // Setup hardware start and stop boundarys based on addressing granularity
    //////////////////////////////////////////////////////////////////////
    BlitX    = (XPos/128) * 128;
    BlitYeven= (YPos & 1) ? YPos+1 : YPos;   // If start is odd line, set to next even
    BlitYodd = (YPos & 1) ? YPos   : YPos+1;
    //             0       128      =127    =0   =1   =128  =127
    //             2       3        =4      =0   =1   =128  =127
    //             2       126      =127    =0   =1   =128  =127
    //             127     1        =127    =0   =1   =128  =127
    //             128     1        =128    =1   =2   =256  =255
    //             128     128      =255    =1   =2   =256  =255
    //             128     129      =256    =2   =3   =384  =383
    //             130     129      =258    =2   =3   =384  =383
    BlitEndX = ((((XPos + BmpSrc->w - 1) / 128) + 1) * 128) - 1;
    BlitW    = (BlitEndX - BlitX) + 1;
    BlitH    = BmpSrc->h;

    OvlBlit->BlitSize = BlitH * BlitW * sizeof(unsigned short);
    if ( ( OvlBlit->YCrCb16Bmp = malloc( OvlBlit->BlitSize ) ) == NULL ) {
      destroy_bitmap( BmpSrc );
      return -1; // If bitmap creation failed, abort.
    }
    
    //////////////////////////////////////////////////////////////////////
    // Copy new (small) sub-Overlay to Index-0 screen overlay
    // The next time GetOverlayBitmap() is called, this new Overlay 0 bitmap
    // will be mixed with the other windows.
    blit(BmpSrc, OvlDesc->Bmp[0], 0, 0, XPos, YPos, BmpSrc->w, BmpSrc->h);
    //
    // void blit(BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height);
    //
    // Copies a rectangular area of the source bitmap to the destination bitmap. 
    // The source_x and source_y parameters are the top left corner of the area 
    // to copy from the source bitmap, and dest_x and dest_y are the corresponding 
    // position in the destination bitmap. This routine respects the destination 
    // clipping rectangle, and it will also clip if you try to blit from areas 
    // outside the source bitmap. 
    //
    // You can blit between any parts of any two bitmaps, even if the two memory 
    // areas overlap (ie. source and dest are the same, or one is sub-bitmap of 
    // the other). You should be aware, however, that a lot of SVGA cards don't 
    // provide separate read and write banks, which means that blitting from one 
    // part of the screen to another requires the use of a temporary bitmap in 
    // memory, and is therefore extremely slow. As a general rule you should avoid 
    // blitting from the screen onto itself in SVGA modes. 
    // 
    // Unlike most of the graphics routines, blit() allows the source and 
    // destination bitmaps to be of different color depths, so it can be used 
    // to convert images from one pixel format to another. 
    
    //////////////////////////////////////////////////////////////////////
    // Perform color conversion and even/odd line packing needed for hardware upload.
    // Hardware blits must also start on 32*4=128 pixel boundarys and
    // widths must be in multiples of 32*4=128 pixels.
    //////////////////////////////////////////////////////////////////////
    OvlBuf = (unsigned short *)OvlBlit->YCrCb16Bmp;
    OvlPixCnt = 0;
    // Transfer pixels to overlay buffer - even lines first.
    for ( LocalY = BlitYeven; LocalY < YPos + BlitH ; LocalY+=2 ) {
        for (LocalX = BlitX ; LocalX < BlitX+BlitW ; LocalX++ ) {
            // get pixel from bitmap.
            pixval = _getpixel16( OvlDesc->Bmp[0], LocalX, LocalY );
            #ifdef YCRCB_OUT
              // convert to YCrCb
              OvlBuf[OvlPixCnt] = RGB2YCrCb16( getr16( pixval ), getg16( pixval ), getb16( pixval ) );
            #else
              // convert to 15-bit RGB with 5-bits for each color.
              OvlBuf[OvlPixCnt] = ( ( pixval & 0xf800 ) >> 1 ) | ( pixval & 0x3ff );  
            #endif //YCRCB_OUT
            
            OvlPixCnt++; // Count the pixels.
        } // Each pixel in line segment
    } // Even Lines

    // Transfer pixels to overlay buffer - odd lines.
    for ( LocalY = BlitYodd; LocalY < YPos + BlitH ; LocalY+=2 ) {
        for (LocalX = BlitX ; LocalX < BlitX+BlitW ; LocalX++ ) {
            // get pixel from bitmap.
            pixval = _getpixel16( OvlDesc->Bmp[0], LocalX, LocalY );
            #ifdef YCRCB_OUT
              // convert to YCrCb
              OvlBuf[OvlPixCnt] = RGB2YCrCb16( getr16( pixval ), getg16( pixval ), getb16( pixval ) );
            #else
              // convert to 15-bit RGB with 5-bits for each color.
              OvlBuf[OvlPixCnt] = ( ( pixval & 0xf800 ) >> 1 ) | ( pixval & 0x3ff );  
            #endif //YCRCB_OUT
            
            OvlPixCnt++; // Count the pixels.
        } // Each pixel in line segment
    } // Odd Lines
    // OvlBlit->YCrCb16Bmp now contains concatinated pixels from the even lines
    // followed by the odd lines of the source image, in 16 bit YCrCb format
    
    //////////////////////////////////////////////////////////////////////
    // Set the rest of the hardware blit registers
    //
    // Assumes OvlDesc->Bmp[0] is first overlay in memory (Address offset 0)
    // Assumes OvlDesc->Bmp[0] has even number of lines
    //////////////////////////////////////////////////////////////////////
    if ((BlitH==1) && (YPos & 1)) { 
      // One line, starting on an odd line number = No even lines exist
      // Put the Start offset in the odd line memory
      OvlBlit->OvlStart     =  ((OvlDesc->Bmp[0]->h/2) * OvlDesc->Bmp[0]->w)         /128 + // Skip even lines
                              (((BlitYodd          /2) * OvlDesc->Bmp[0]->w) + BlitX)/128;
    } else {
      OvlBlit->OvlStart     = (((BlitYeven         /2) * OvlDesc->Bmp[0]->w) + BlitX)/128;
    }
    // OvlMaxCol =           127        0    =127 =0
    //                       255       -128  =127 =0
    //                       255       -0    =255 =1
    OvlBlit->OvlMaxCol    = ((BlitEndX - BlitX )/128); // Number of 128 pix groups minus one each line
    // BlitYeven-YPos = 0 when starts on even
    // BlitYeven-YPos = 1 when starts on odd
    // If odd num lines, the extra line is either even or odd. If start=even, add one to even
    OvlBlit->OvlMaxRow    = (BlitH/2) + (((BlitH & 1) && (BlitYeven==YPos)) ? 1 : 0); 
    OvlBlit->OvlMaxRow    = OvlBlit->OvlMaxRow - 1;// Number of Even lines minus one.
    // Calc increment that will increment a running address counter enough to:
    // at BlitEndX
    // (BlitEndX + OvlDesc->Bmp[0]->w  - BlitEndX + BlitX)/128
    // (383      + 640                 - 383      + 0)/128
    // 383+640=1023 -383=640+0=640/128=5
    // 383/128   + 640/128             - 383/128  + 0/128
    // 2         + 5                   -2         + 0 = 5
    OvlBlit->OvlNextLine  = 
       (   OvlDesc->Bmp[0]->w/128 //  put it on the next line, in groups of 128 pix
         - OvlBlit->OvlMaxCol );  // back it up to the beginning of the line (since the counter is at BlitEndX when it is updated)
    // At the point OvlNextField is used, the counter will be at the last (128 group) pix
    // on the last even line. So, 
    OvlBlit->OvlNextField = 
       (  (OvlDesc->Bmp[0]->h /2  * OvlDesc->Bmp[0]->w ) // Increment past even lines into odd lines
        + (OvlBlit->OvlNextLine*128) // increment to the start of the next line
        - ((OvlBlit->OvlMaxRow+1) * OvlDesc->Bmp[0]->w) // Back up by number of even lines
        - ((BlitYeven         /2) * OvlDesc->Bmp[0]->w) // Back up by start of even lines
        + ((BlitYodd          /2) * OvlDesc->Bmp[0]->w) // Start at the first odd line
       )  /128;                  // In groups of 128 pix

    destroy_bitmap( BmpSrc );

    // Return success or failure.
    return 0; // Success if this is reached
}

///////////////////////////////////////////////////////////////////////
// Gets overlay buffer. This buffer is sent to overlay hardware.
//
// OvlHandle    - Handle from CreateOverlay().
// Bytes        - Pointer to int that is to receive the number of
//                bytes in overlay buffer. Use NULL if not interested
//                in byte count.
// imageWidth   - width of the frame, for clipping
// imageHeight  - height of the frame, for clipping
//
// Returns pointer to overlay buffer on success, null on failure. 
//
OVLBUFPTR DLLEXP GetOverlayBitmap( OVLHAN OvlHandle, int *Bytes, int imageWidth, int imageHeight, int Interlaced )
{
    OVLDESC         *OvlDesc = (OVLDESC *)OvlHandle;
    BITMAP          *CurBmp;
    unsigned short  *OvlBuf;
    int             i, j;
    int             LineSegNum;
    int             NumLines;
    int             NumPixels;
    int             LineCnt;
    int             PixCnt;
    int             OvlPixCnt;
    LINESEG        *LineSeg;
    int             pixval, ovlpix;
    int             CurPos;
    int             LocalX;
    int             LocalY;
    int             EvenOdd;

    if ( !OvlDesc )
        return NULL;

    OvlBuf = OvlDesc->Bitmap;
    
    // Count lines and pixels.
    for ( i = 0, NumLines = 0, NumPixels = 0; i < OVLWINDOWS; i++ )
    {
        if ( OvlDesc->Bmp[i] )
        {
            int xpos, width;
            xpos = OvlDesc->XPos[i];
            width = OvlDesc->Bmp[i]->w;
                
            if ( xpos < imageWidth && xpos + width > 0 )
            {
                if ( xpos < 0 ) { width += xpos; xpos = 0; }
                if ( xpos + width > imageWidth ) { width = imageWidth - xpos; }
            
                for ( LineCnt = 0; LineCnt < OvlDesc->Bmp[i]->h; LineCnt++ )
                    if ( OvlDesc->YPos[i] + LineCnt >= 0 && OvlDesc->YPos[i] + LineCnt < imageHeight )
                    { 
                        NumLines++;
                        NumPixels+=width;
                    } 
            }
            //NumLines  += OvlDesc->Bmp[i]->h;
            //NumPixels += OvlDesc->Bmp[i]->h * OvlDesc->Bmp[i]->w;
            //Debug
            //printf( "W:%d H:%d\n", Bmp[i]->w, Bmp[i]->h );
            //char bitfile[80];
            //sprintf( bitfile, "bitmap%d.bmp", i );
            //save_bmp( bitfile, Bmp[i], NULL );
        }
    }

    // Make sure we haven't exceeded our maximum allowed pixels.
    if ( NumPixels > gMaxPix || NumPixels == 0 )
        return NULL;
    
    // Allocate segment buffer.
    if ( ( LineSeg = (LINESEG *) malloc( NumLines * sizeof(LINESEG) ) ) == NULL )
        return NULL;

    // Put each line from all bitmaps into a segment buffer.
    for ( i = 0, LineSegNum = 0; i < OVLWINDOWS; i++ )
    {
        if ( OvlDesc->Bmp[i] )
        {
            int xpos, width;
            xpos = OvlDesc->XPos[i];
            width = OvlDesc->Bmp[i]->w;
                
            if ( xpos < imageWidth && xpos + width > 0 )
            {
                if ( xpos < 0 ) { width += xpos; xpos = 0; }
                if ( xpos + width > imageWidth ) { width = imageWidth - xpos; }
            
                for ( LineCnt = 0; LineCnt < OvlDesc->Bmp[i]->h; LineCnt++ )
                    if ( OvlDesc->YPos[i] + LineCnt >= 0 && OvlDesc->YPos[i] + LineCnt < imageHeight )
                    {   
                        // A relative position of the segment.
                        LineSeg[LineSegNum].OffsetStart = ((OvlDesc->YPos[i] + LineCnt)*2048) + xpos;
                        LineSeg[LineSegNum].OffsetEnd   = ((OvlDesc->YPos[i] + LineCnt)*2048) + xpos + width - 1;
                        // Y position of segment.
                        LineSeg[LineSegNum].FrameY      = OvlDesc->YPos[i] + LineCnt;
                        // Line number within bitmap of segment.
                        LineSeg[LineSegNum].LocalY      = LineCnt;
                        // Starting column within bitmap of segment.
                        LineSeg[LineSegNum].LocalX      = xpos - OvlDesc->XPos[i];
                        // The ovlerlay window this line segment comes from
                        LineSeg[LineSegNum].SubWin      = i;
                        // Set text or image type
                        LineSeg[LineSegNum].Type        = OvlDesc->Type[i]; // 0=Text, 1=Image
                        // Bitmap.
                        LineSeg[LineSegNum].Bmp         = OvlDesc->Bmp[i]; // Bmp[i];
                        // Count the number of segments. 
                        LineSegNum++;
                    }
            }
        }
    }
    ASSERT (NumLines == LineSegNum);

    // Sort segment positions using Quicksort algorithm.
    qsort( (void *)LineSeg, (size_t)LineSegNum, sizeof( LINESEG ), ComparePos );

    OvlPixCnt = 0;
    for (EvenOdd = 0; EvenOdd <= Interlaced ; EvenOdd++) { // Only one pass for progressive
        CurPos = LineSeg[0].OffsetStart;
        // Transfer segments to overlay buffer - first even, then odd lines.
        for ( i = 0; i < LineSegNum; i++ )
        {
            if ( ( ( LineSeg[i].FrameY & 1 ) == EvenOdd ) || (Interlaced == 0) ) // every time for progressive
            {
                // If start of new line, CurPos will be set to this lines segments OffsetStart
                // If this line segment is on the same line as the previous line segment,
                // this line may have overlapped the last line and CurPos will have advanced
                // beyond the start of the line.
                if (CurPos < LineSeg[i].OffsetStart) {
                    CurPos = LineSeg[i].OffsetStart; // New line starts past last lines last pix
                    PixCnt = 0;
                } else {
                    PixCnt = CurPos - LineSeg[i].OffsetStart;  // If were already past this line seg,
                }
                //for ( ; PixCnt < LineSeg[i].Bmp->w; PixCnt++ ) // we will fall though this loop
                for ( ; CurPos <= LineSeg[i].OffsetEnd; PixCnt++ ) // we will fall though this loop
                {
                    CurBmp = LineSeg[i].Bmp;
                    LocalX = LineSeg[i].LocalX + CurPos - LineSeg[i].OffsetStart;
                    LocalY = LineSeg[i].LocalY;
        
                    // get pixel from bitmap.
                    pixval = _getpixel16( CurBmp, LocalX, LocalY );
        
                    // Collision detect
                    for (j = 1; j < OVLWINDOWS ; j++) { // max of OVLWINDOWS-1 more lines on top of this
                        if ((i+j < LineSegNum) &&
                            (LineSeg[i+j].FrameY == LineSeg[i].FrameY)) {
                            // Same video line
                            if ((CurPos >= LineSeg[i+j].OffsetStart) && // Overlap
                                (CurPos <= LineSeg[i+j].OffsetEnd ) &&
                                 (LineSeg[i+j].SubWin > LineSeg[i].SubWin)) { // Only sub-windows with bigger indexes are on top
                                // Overlap it
                                CurBmp = LineSeg[i+j].Bmp;
                                LocalX = LineSeg[i+j].LocalX + CurPos - LineSeg[i+j].OffsetStart;
                                LocalY = LineSeg[i+j].LocalY;
        
                                ovlpix = _getpixel16( CurBmp, LocalX, LocalY );
                                if ((ovlpix != 0) || (LineSeg[i+j].Type == 0)) { 
                                    // Text or Not transparent
                                    // (use transparent text so background window will show through)
                                    pixval = ovlpix; 
                                }
                            }
                        } else {
                            break;
                        }
                    }
        
                    //// get pixel from bitmap.
                    //pixval = _getpixel16( CurBmp, LocalX, LocalY );
        
                    #ifdef YCRCB_OUT
                      // convert to YCrCb
                      OvlBuf[OvlPixCnt] = RGB2YCrCb16( getr16( pixval ), getg16( pixval ), getb16( pixval ) );
                    #else
                      // convert to 15-bit RGB with 5-bits for each color.
                      OvlBuf[OvlPixCnt] = ( ( pixval & 0xf800 ) >> 1 ) | ( pixval & 0x3ff );  
                    #endif //YCRCB_OUT
        
                    OvlPixCnt++; // Count the pixels.
                    CurPos++;    // Track current X position in virtual frame
                } // Each pixel in line segment
            } // Curline == EvenOdd
            if (i < LineSegNum-1) {
                if (LineSeg[i+1].FrameY != LineSeg[i].FrameY) { // First new video line
                    CurPos = LineSeg[i+1].OffsetStart; // Set next line as first point
                }
                if ( Interlaced && ((LineSeg[i].FrameY & 1 ) != EvenOdd)) { // Cur line just skipped?
                    CurPos = LineSeg[i+1].OffsetStart; // Set next line as first point
                }
            }
            // else on same line, keep CurPos
        } // For each LineSeg[]
    } // EvenOdd

    // Return number of bytes in overlay buffer.
    if ( Bytes )
        *Bytes = OvlPixCnt * 2;

    // Free the segment buffer.
    free( LineSeg );

    // Return pointer to overlay buffer.
    return OvlBuf;
}

///////////////////////////////////////////////////////////////////////
// Get overlay sub-window width
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
//
// Returns width on success, -1 on failure. 
//
int DLLEXP GetOverlayWidth( OVLHAN OvlHandle, int WinIndex )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS || !OvlDesc->Bmp[WinIndex] )
        return -1;

    return OvlDesc->Bmp[WinIndex]->w;
}

///////////////////////////////////////////////////////////////////////
// Get overlay sub-window Height
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - OVLWINDOWS-1.
//
// Returns height on success, -1 on failure. 
//
int DLLEXP GetOverlayHeight( OVLHAN OvlHandle, int WinIndex )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;

    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS || !OvlDesc->Bmp[WinIndex] )
        return -1;

    return OvlDesc->Bmp[WinIndex]->h;
}


///////////////////////////////////////////////////////////////////////
// Destroys the overlay system.
//
// OvlHandle    - Handle from CreateOverlay().
//
void DLLEXP DestroyOverlay( OVLHAN OvlHandle )
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    int i;

    if ( !OvlDesc )
        return;

    for ( i = 0; i < OVLWINDOWS; i++ )
    {
        OvlDesc->Type[i] =  1; // 0=Text, 1=Image
        if ( OvlDesc->Bmp[i] ) {
            destroy_bitmap( OvlDesc->Bmp[i] );
        }
        if ( OvlDesc->Font[i] ) {
            alfont_destroy_font( OvlDesc->Font[i] );
        }
        if ( OvlDesc->rgb24Ptr[i] ) {
            free( OvlDesc->rgb24Ptr[i] );
            OvlDesc->rgb24Ptr[i] = 0;
        }

    }

    free( OvlDesc );
}



#define COLOR_DEPTH_RGB24 24
#define BYTES_PER_PIXEL24 (((int)(COLOR_DEPTH_RGB24) + 7) / 8)

///////////////////////////////////////////////////////////////////////
// Gets overlay sub-window RGB bitmap. 
//    This is a raw (8 bit Red) (8 bit Green) (8 bit Blue) array of
//    bytes defineing the bitmap in the given sub-window.
//
// Input
//    OvlHandle  - Handle from CreateOverlay().
//    flip       - If true, the returned buffer will be mirrored vertically.
//    WinIndex   - Overlay sub-window, 0 to OVLWINDOWS-1.
//    backcolor  - If backcolor != 0, replace source with this if source = 0
// Returns 
//    ppBuffer24 - Sets *ppBuffer24 to RGB24 pointer containing bitmap
//                 This routine will allocate
//    pSize      - size in bytes of ppBuffer24
// Returns null on failure. 
//
int DLLEXP GetOverlayBitmap24( OVLHAN OvlHandle, int flip, int WinIndex, int backcolor, char **ppBuffer24, int *pSize)
{
    OVLDESC *OvlDesc = (OVLDESC *)OvlHandle;
    BITMAP  *pBitMap24; // intermediate bitmap for converting to 24 bit RGB
    char    *dptr, *sptr;
    int      Buf24Size, Buf24SizeW, Buf24LastLine;
    int      ii,x;
    
    *pSize = 0;
    
    // Make sure the overlay system has been created and the sub-window is valid.
    if ( !OvlDesc || WinIndex < 0 || WinIndex >= OVLWINDOWS )
        return 0;

    // If a bitmap already exists, destroy it so that we can create a new one.
    if ( OvlDesc->Bmp[WinIndex] == 0)
        return 0;

    // Make sure the return pointer is valid.
    if ( ppBuffer24==0 )
        return 0;

    *ppBuffer24 = 0;

    Buf24SizeW = BYTES_PER_PIXEL24 * OvlDesc->Bmp[WinIndex]->w;
    Buf24Size  = Buf24SizeW        * OvlDesc->Bmp[WinIndex]->h;
    if ( OvlDesc->rgb24Ptr[WinIndex] ) {
        free( OvlDesc->rgb24Ptr[WinIndex] );
        OvlDesc->rgb24Ptr[WinIndex] = 0;
    }
    OvlDesc->rgb24Ptr[WinIndex] = (char *)malloc( Buf24Size );
    if (! OvlDesc->rgb24Ptr[WinIndex]) 
        return(0);

    // Create 24 bit (blit will automatically convert to this format from the original format)
    pBitMap24 = create_bitmap_ex(COLOR_DEPTH_RGB24, OvlDesc->Bmp[WinIndex]->w, OvlDesc->Bmp[WinIndex]->h);
    if (!pBitMap24) 
        return(0);

    clear_bitmap( pBitMap24 );

    // Convert 16 bit color bitmap to 24 bit for DV RGB24 screen overlay
    blit(OvlDesc->Bmp[WinIndex], pBitMap24, 0, 0, 0, 0, pBitMap24->w, pBitMap24->h);

    char backcolor_red = (backcolor >> 0) & 0xFF;
    char backcolor_grn = (backcolor >> 8) & 0xFF;
    char backcolor_blu = (backcolor >>16) & 0xFF;

    if (flip != 0) {
        Buf24LastLine = Buf24SizeW * (OvlDesc->Bmp[WinIndex]->h - 1);
        for (ii=0; ii < pBitMap24->h ; ii++) {
            dptr = OvlDesc->rgb24Ptr[WinIndex] + Buf24LastLine - (Buf24SizeW*ii); // Dest=dec down from Last line 
            sptr = ((char*)pBitMap24->dat)                   + (Buf24SizeW*ii); // Source increment up
            for (x=0 ; x < Buf24SizeW ; x+=3) {
                if ((*(sptr+0)==0 && *(sptr+1)==0 && *(sptr+2)==0) && (backcolor != 0)) { 
                    // Source=Background, Set background color
                    *(dptr+0)=backcolor_blu; *(dptr+1)=backcolor_grn;  *(dptr+2)=backcolor_red;
                } else { // Will be Transparent if = 0
                    // flip Red/Blue
                    *(dptr+0)=*(sptr+2);     *(dptr+1)=*(sptr+1);      *(dptr+2)=*(sptr+0);
                }
                dptr += BYTES_PER_PIXEL24; // 3
                sptr += BYTES_PER_PIXEL24;
            }
        }
    } else { // NO Flip
        for (ii=0; ii < pBitMap24->h ; ii++) {
            dptr = OvlDesc->rgb24Ptr[WinIndex] + (Buf24SizeW*ii); // Dest=inc up from first line 
            sptr = ((char*)pBitMap24->dat)   + (Buf24SizeW*ii); // Source increment up
            for (x=0 ; x < Buf24SizeW ; x+=3) {
                if ((*(sptr+0)==0 && *(sptr+1)==0 && *(sptr+2)==0) && (backcolor != 0)) { 
                    // Source=Background, Set background color
                    *(dptr+0)=backcolor_red; *(dptr+1)=backcolor_grn;  *(dptr+2)=backcolor_blu;
                } else { // Will be Transparent if = 0
                    // straight copy
                    *(dptr+0)=*(sptr+0);     *(dptr+1)=*(sptr+1);      *(dptr+2)=*(sptr+2);
                }
                dptr += BYTES_PER_PIXEL24; // 3
                sptr += BYTES_PER_PIXEL24;
            }
        }
    }

    *ppBuffer24 = OvlDesc->rgb24Ptr[WinIndex];
    *pSize = Buf24Size;
    
    // Free the intermediate bitmap.
    destroy_bitmap( pBitMap24 );

    return -1;
}

void DLLEXP SetOverlayBitmapSize(int DoubleSize )
{
#ifdef OVL_IN_DDR
   gMaxPix =   OVLMAXPIX; // 1920*1080 = 2073600, 16-bit pix
#else
   if (DoubleSize) {
      gMaxPix = 2*OVLMAXPIX; // 65536 9-bit pix
   } else {
      gMaxPix =   OVLMAXPIX; // 32768 16-bit pix
   }
#endif
}


void DLLEXP ShutdownOverlay(  )
{
    alfont_exit();
    allegro_exit();
    bOverlayInit = 0;
}

