#ifndef _OVLGEN_H_
#define _OVLGEN_H_


#define MAX_TEXTSIZE 256
#define DLLEXP

// Number of overlay Sub-Windows. Dictated by hardware.
#define OVLWINDOWS	8

#define OVL_IN_DDR

#ifdef OVL_IN_DDR
// Size of hardware overlay buffer in DDR memory.
#define OVLMAXPIX       16777216 //plenty, Each pix is 16 bits, 1920*1080 = 2073600
#define OVLMAXPIXBUFSZ  2073608 // Max 1920x1080 image (plus a couple for the flush word)
#else
// Size of hardware overlay buffer.
#define OVLMAXPIX       32768 // Each pix is 16 bits
#define OVLMAXPIXBUFSZ  65536 // Twice as much when in 9-bit color mode
#endif

// Overlay handle type
typedef void * OVLHAN;
typedef void * OVLBUFPTR;

#ifndef SENSORAY_OS_LINUX
#ifdef __cplusplus
extern "C" {
#endif
#endif

// Overlay description structure. 
typedef struct _OVLBLIT
{
    int             OvlStart; 
    int             OvlMaxCol;
    int             OvlMaxRow;
    int             OvlNextLine;
    int             OvlNextField;
    int             BlitSize;
    OVLBUFPTR       YCrCb16Bmp; // void * to unsigned short array of Y(15:CrCb
} OVLBLIT;

///////////////////////////////////////////////////////////////////////
// Create overlay system. This must be called first as it returns a
// handle used by all the other overlay functions.
//
// Interlaced	- Non-zero if overlay buffer is interlaced.
//
// Returns handle on success, null on failure. 
//
OVLHAN DLLEXP CreateOverlay( int Interlaced );

///////////////////////////////////////////////////////////////////////
// Delete overlay sub-window position
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay window, 0 to OVLWINDOWS-1.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP DelOverlayIndex( OVLHAN OvlHandle, int WinIndex);
 
///////////////////////////////////////////////////////////////////////
// Set overlay sub-window position. The values are the same as the values
// given to the hardware overlay generator.
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
// XPos			- Horizontal pixel position from left.
// YPos			- Vertical pixel position from top.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayPosition( OVLHAN OvlHandle, int WinIndex, int XPos, int YPos );

///////////////////////////////////////////////////////////////////////
// Set overlay sub-window font. Call before SetOverlayText() if font
// change desired.
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
// FontFile		- Full name/path to font file.
// FontSize		- Height of font in pixels.
// Antialiased	- Non-zero means antialiased.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayFont( OVLHAN OvlHandle, int WinIndex, char *FontFile, int FontSize, int Antialiased );

///////////////////////////////////////////////////////////////////////
// Set overlay sub-window text colors. Call before SetOverlayText() if
// color change desired.
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
// Color		- Text color.
// BkgColor		- Background color, zero or less means transparent.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayTextColor( OVLHAN OvlHandle, int WinIndex, int Color, int BkgColor );

///////////////////////////////////////////////////////////////////////
// Raterize text to an overlay sub-window.
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
// Text			- Null terminated string.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayText( OVLHAN OvlHandle, int WinIndex, char *Text );

///////////////////////////////////////////////////////////////////////
// Place an image in an overlay sub-window.
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
// ImageFile	- Full name/path to image file. 24-bit bmp and pcx 
//				  files are supported.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayImage( OVLHAN OvlHandle, int WinIndex, char *ImageFile );

///////////////////////////////////////////////////////////////////////
// Texture map an image onto a set of 4 coordinates
//
// OvlHandle    - Handle from CreateOverlay().
// WinIndex     - Overlay sub-window, 0 - (OVLWINDOWS-1)
// ImageFile    - Full name/path to image file. 24-bit bmp and pcx 
//                files are supported.
// x1, y1, x2, y2, x3, y3, x4, y4 - Coordinates of the polygon, where
//				- (x1,y1) is upper left corner of the texture 
//				- (x2,y2) is upper right corner of the texture 
//				- (x3,y3) is lower right corner of the texture 
//				- (x4,y4) is lower left corner of the texture 
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP SetOverlayQuad( OVLHAN OvlHandle, int WinIndex, char* ImageFile, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4 );

///////////////////////////////////////////////////////////////////////
// Blit new sub-window to image in overlay zero and calculates values 
// needed by hardware to quickly blit minimal new sub-window data onto overlay
// in hardware that is organized as larger set of even rows followed by odd rows.
// Assumes OvlBmp[0] is loaded into current memory AND double buffer. (Loaded twice)
// Assumes OvlBmp[0] is first overlay in memory (Address offset 0)
// Assumes OvlBmp[0] has even number of lines
// Assumes OvlBmp[0] has a width that is divisible by 128
// Assumes all other overlays (OvlBmp[1-7] are contained within the
// boundaries of OvlBmp[0]
//
// OvlHandle    - Handle from CreateOverlay().
// OvlBlit      - Pointer to structure that returns with hardware values needed for immediate 'quick' copy.
// ImageFile    - Full name/path to image file. 24-bit bmp and pcx 
//                files are supported.
// XPos         - Horizontal Destination pixel position from left to copy bitmap to.
// YPos         - Vertical Destination pixel position from top to copy bitmap to.
//
// Returns 0 on success, -1 on failure. 
//
int DLLEXP CopyBmpToOverlayZero( OVLHAN OvlHandle, OVLBLIT *OvlBlit, char *ImageFile, int XPos, int YPos );

///////////////////////////////////////////////////////////////////////
// Gets overlay buffer. This buffer is sent to overlay hardware.
//
// OvlHandle	- Handle from CreateOverlay().
// Bytes		- Pointer to int that is to receive the number of
//				  bytes in overlay buffer. Use NULL if not interested
//				  in byte count.
// imageWidth	- width of the frame, for clipping
// imageHeight  - height of the frame, for clipping
//
// Returns pointer to overlay buffer on success, null on failure. 
//
OVLBUFPTR DLLEXP GetOverlayBitmap( OVLHAN OvlHandle, int *Bytes, int imageWidth, int imageHeight, int Interlaced);

///////////////////////////////////////////////////////////////////////
// Get overlay sub-window width
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
//
// Returns width on success, -1 on failure. 
//
int DLLEXP GetOverlayWidth( OVLHAN OvlHandle, int WinIndex );

///////////////////////////////////////////////////////////////////////
// Get overlay sub-window Height
//
// OvlHandle	- Handle from CreateOverlay().
// WinIndex		- Overlay sub-window, 0 to OVLWINDOWS-1.
//
// Returns height on success, -1 on failure. 
//
int DLLEXP GetOverlayHeight( OVLHAN OvlHandle, int WinIndex );

///////////////////////////////////////////////////////////////////////
// Gets overlay sub-window RGB bitmap. 
//    This is a raw (8 bit Red) (8 bit Green) (8 bit Blue) array of
//    bytes defineing the bitmap in the given sub-window.
//
// Input
//    OvlHandle  - Handle from CreateOverlay().
//    flip       - If true, the returned buffer will be mirrored vertically.
//    WinIndex   - Overlay sub-window, 0 to OVLWINDOWS-1.
//    backcolor  - If backcolor != 0, replace source with this if source = 0
// Returns 
//    ppBuffer24 - Sets *ppBuffer24 to RGB24 pointer containing bitmap
//                 This routine will allocate
//    pSize      - size in bytes of ppBuffer24
// Returns null on failure. 
//
int DLLEXP GetOverlayBitmap24( OVLHAN OvlHandle, int flip, int WinIndex, int backcolor, char **ppBuffer24, int *pSize);

///////////////////////////////////////////////////////////////////////
// Destroys the overlay system.
//
// OvlHandle	- Handle from CreateOverlay().
//
void DLLEXP DestroyOverlay( OVLHAN OvlHandle );

///////////////////////////////////////////////////////////////////////
// Destroys the overlay system.
//
// DoubleSize	- 0= 32768  max pixel overlay (for 16-bit pixels).
//              1= 65536  max pixel overlay (for 9-bit pixels).
//
void DLLEXP SetOverlayBitmapSize(int DoubleSize );


void DLLEXP ShutdownOverlay();

#ifndef SENSORAY_OS_LINUX
#ifdef __cplusplus
}
#endif
#endif

#endif //_OVLGEN_H_
